SystemOrganization addCategory: #SortDescriptors!

Object subclass: #ASortDescriptor
	instanceVariableNames: 'ascending subSortDescriptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!ASortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
ascending
	^ ascending! !

!ASortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
ascending: aBoolean
	ascending := aBoolean! !

!ASortDescriptor methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:26'!
changeDirection

	ascending := ascending not! !

!ASortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:27'!
subSortDescriptor
	^ subSortDescriptor! !

!ASortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:27'!
subSortDescriptor: anObject
	subSortDescriptor := anObject! !

ASortDescriptor subclass: #BlockSortDescriptor
	instanceVariableNames: 'aBlock dBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!BlockSortDescriptor classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:22'!
classHierarchySortingDescriptor

	|instance|
	instance := self new.
	instance descent: [:class1 :class2| class1 inheritsFrom: class2].
	instance ascent: [:class1 :class2| class2 inheritsFrom: class1].
	^instance! !

!BlockSortDescriptor methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:22'!
ascent: aBlockClosure

	self assert: [ aBlockClosure numArgs = 2 ].
	aBlock := aBlockClosure! !

!BlockSortDescriptor methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:22'!
descent: aBlockClosure

	self assert: [ aBlockClosure numArgs = 2 ].
	dBlock := aBlockClosure! !

!BlockSortDescriptor methodsFor: 'evaluating' stamp: 'hjo 1/8/2012 14:43'!
value: object1 value: object2

	(dBlock value: object1 value: object2)
		ifTrue: [ ^ascending = false ].
	(aBlock value: object1 value: object2)
		ifTrue: [ ^ascending = true ].
	^subSortDescriptor isNil 
		ifTrue: [ true ]
		ifFalse: [ subSortDescriptor value: object1 value: object2 ]! !

ASortDescriptor subclass: #SortDescriptor
	instanceVariableNames: 'aspect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!SortDescriptor commentStamp: 'hjo 1/8/2012 15:29' prior: 0!
(NOTE) I will be renamed to AspectSortDescriptor, and superclass will take my name.

I can work as a sort-block of SortedCollection.

Smalltalk sort-blocks have two arguments and describes boolean expression that defines the sorting.
I am instanticated differently. You specify the sorting direction and aspect. An aspect is used to access value from the objects to be sorted.

For example,
anArray sorted: (SortDescriptor ascendingBy: #price).

Aspect is can be symbol of one argument or block of one argument. Above example can be rewritten with block like below.

anArray sorted: (SortDescriptor ascendingBy: [:obj| obj price]).

In addition to that, I can change ascending/descending mode. So it is very easy thing to toggle sorting direction in the list view.

sd := SortDescriptor ascendingBy: #price.
anArray sorted: sd. "ascending sort"
sd ascending: false.
anArray sorted: sd. "descending sort"


In addition to that, I support multi-sorting by using subSortDescriptor.  
anArray sorted: (SortDescriptor ascendingBy: #price subSortDescriptor: (SortDescriptor descendingBy: [:x| (x perform: #parent) name]) )!

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 19:54'!
ascendingBy: aSymbol 

	^self new 
		aspect: aSymbol; 
		ascending: true! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 20:37'!
ascendingBy: aSymbol ascendingBy: aSymbol2 ascendingBy: aSymbol3 

	^self ascendingBy: aSymbol subSortDescriptor: (self ascendingBy: aSymbol2 subSortDescriptor: (self ascendingBy: aSymbol3)).! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 19:55'!
ascendingBy: aSymbol subSortDescriptor: aSortDescriptor 

	^(self ascendingBy: aSymbol)
		subSortDescriptor: aSortDescriptor   ! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 20:36'!
descendingBy: aSymbol 

	^self new 
		aspect: aSymbol; 
		ascending: false! !

!SortDescriptor class methodsFor: 'instance creation' stamp: 'hjo 11/3/2011 20:36'!
descendingBy: aSymbol subSortDescriptor: aSortDescriptor 

	^(self descendingBy: aSymbol)
		subSortDescriptor: aSortDescriptor   ! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 1/8/2012 15:26'!
aspect
	^aspect! !

!SortDescriptor methodsFor: 'accessing' stamp: 'hjo 11/3/2011 19:36'!
aspect: aSymbol
	aspect := aSymbol! !

!SortDescriptor methodsFor: 'evaluating' stamp: 'hjo 11/3/2011 20:32'!
value: anObject1 value: anObject2 

	| v1 v2 |
	v1 := aspect value: anObject1.
	v2 := aspect value: anObject2.
	^v1 = v2
		ifTrue: [ subSortDescriptor isNil or: [ subSortDescriptor value: anObject1 value: anObject2 ] ]
		ifFalse: [ ascending xor: [v1 > v2] ]! !

TestCase subclass: #ClassHierarchySortDescriptorTest
	instanceVariableNames: 'objects sortedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 13:52'!
printedStringOfSortedObjects

	^String streamContents: [:stream|
		sortedObjects do: [:each| stream print: each;cr]
		]  
	! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 14:03'!
setUp

	objects := {
		Collection.
		String.
		Object.
		Array.
		}.! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 13:52'!
sortWithDescriptor: aSortDescriptor 

	sortedObjects := objects sorted: aSortDescriptor! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:10'!
testAscending

	self sortWithDescriptor: (BlockSortDescriptor classHierarchySortingDescriptor ascending: true).
	self assert: 4 equals: sortedObjects size. 
	self assert: 
'Object
Collection
String
Array
'	equals: self printedStringOfSortedObjects! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:10'!
testCollectionVsObject

	|sd|
	sd := (BlockSortDescriptor classHierarchySortingDescriptor ascending: true).
	self deny: (sd value: Collection value: Object). 
! !

!ClassHierarchySortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 1/8/2012 15:09'!
testObjectVsCollection

	|sd|
	sd := (BlockSortDescriptor classHierarchySortingDescriptor ascending: true).
	self assert: (sd value: Object value: Collection).
! !

TestCase subclass: #SortDescriptorTest
	instanceVariableNames: 'objects sortedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SortDescriptors'!

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:19'!
printedStringOfSortedObjects

	^String streamContents: [:stream|
		sortedObjects do: [:each| stream print: each;cr]
		]  
	! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:11'!
setUp

	objects := {
		{100. 'Astachio'}. 
		{1. 'Betty'}. 
		{60. 'Carl'}. 
		{1. 'Duffy'}. 
		{10. 'Elizabeth'}.
		{60. 'Felix'}. 
		{5. 'Gerero'}. 
		{60. 'Howard'}. 
		{1. 'Isabell'}. 
		{6. 'Julian'}.
		}.! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:15'!
sortWithDescriptor: aSortDescriptor 

	sortedObjects := objects sorted: aSortDescriptor! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:20'!
testAscending

	self sortWithDescriptor: (SortDescriptor ascendingBy: #first).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:21'!
testBlockAspect

	self sortWithDescriptor: (SortDescriptor ascendingBy: [:each| each first]).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:25'!
testChangeDirection

	| sd |
	sd := SortDescriptor ascendingBy: #first.
	self sortWithDescriptor: sd.
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects.
	sd changeDirection.
	self sortWithDescriptor: sd.
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(100 ''Astachio'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(10 ''Elizabeth'')
#(6 ''Julian'')
#(5 ''Gerero'')
#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
' equals: self printedStringOfSortedObjects.
	! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:21'!
testDescending

	self sortWithDescriptor: (SortDescriptor descendingBy: #first).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(100 ''Astachio'')
#(60 ''Carl'')
#(60 ''Felix'')
#(60 ''Howard'')
#(10 ''Elizabeth'')
#(6 ''Julian'')
#(5 ''Gerero'')
#(1 ''Betty'')
#(1 ''Duffy'')
#(1 ''Isabell'')
' equals: self printedStringOfSortedObjects! !

!SortDescriptorTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/3/2011 21:22'!
testSubDescriptor

	self sortWithDescriptor: (SortDescriptor ascendingBy: #first subSortDescriptor: (SortDescriptor descendingBy: #second)).
	self assert: 10 equals: sortedObjects size. 
	self assert: '#(1 ''Isabell'')
#(1 ''Duffy'')
#(1 ''Betty'')
#(5 ''Gerero'')
#(6 ''Julian'')
#(10 ''Elizabeth'')
#(60 ''Howard'')
#(60 ''Felix'')
#(60 ''Carl'')
#(100 ''Astachio'')
' equals: self printedStringOfSortedObjects! !
